@@ -5,11 +5,6 @@ from typing import TYPE_CHECKING, Union, Tuple, Sequence
 from numpy.typing import ArrayLike
 if TYPE_CHECKING:
     from neurograd.tensor import Tensor
-try:
-    import opt_einsum
-except ImportError:
-    opt_einsum = None
-
 
 def _transpose(arr):
     """Transpose that works for any ndim >= 1"""
@@ -113,33 +108,17 @@ class TensorDot(Function, Module):
             grad_B = xp.transpose(grad_B, perm) 
         return grad_A, grad_B
 
-
 class EinSum(Function, Module):
     name = "EinSum"
-    def __init__(self, subscripts: str, optimize=False):
+    def __init__(self, subscripts: str, optimize = False):
         Function.__init__(self)
         Module.__init__(self)
         self.subscripts = subscripts.replace(" ", "")
         self.optimize = optimize
-        self.forward_path = None
-        self.backward_paths = {}
     def forward(self, *operands: xp.ndarray) -> xp.ndarray:
         self.operand_shapes = [op.shape for op in operands]
         self.input_operands = operands
-        if opt_einsum is not None and self.optimize:
-            if self.forward_path is None:
-                dummy_arrays = [xp.zeros(shape) for shape in self.operand_shapes]
-                # contract_path returns (path, path_info)
-                self.forward_path, _ = opt_einsum.contract_path(
-                    self.subscripts, *dummy_arrays, optimize='auto'
-                )
-            return opt_einsum.contract(
-                self.subscripts, *operands, 
-                backend='cupy' if xp.__name__ == 'cupy' else 'numpy',
-                optimize=self.forward_path
-            )
-        else:
-            return xp.einsum(self.subscripts, *operands, optimize=self.optimize)
+        return xp.einsum(self.subscripts, *operands, optimize=self.optimize)   
     def backward(self, grad_output: xp.ndarray):
         grads = []
         if '->' in self.subscripts:
@@ -148,11 +127,13 @@ class EinSum(Function, Module):
         else:
             input_specs = self.subscripts.split(',')
             all_indices = ''.join(input_specs)
-            output_str = ''.join([idx for idx in set(all_indices) if all_indices.count(idx) == 1])
+            output_str = ''.join(sorted(set(all_indices) - 
+                                      set([idx for idx in all_indices 
+                                           if all_indices.count(idx) > 1])))
         for i, tensor in enumerate(self.parent_tensors):
             if not tensor.requires_grad:
                 grads.append(None)
-                continue 
+                continue
             grad_operands = [grad_output]
             grad_specs = [output_str]
             for j, operand in enumerate(self.parent_tensors):
@@ -160,27 +141,12 @@ class EinSum(Function, Module):
                     grad_operands.append(operand.data)
                     grad_specs.append(input_specs[j])
             grad_equation = ','.join(grad_specs) + '->' + input_specs[i]
-            if opt_einsum is not None and self.optimize:
-                if grad_equation not in self.backward_paths:
-                    dummy_shapes = [grad_output.shape]
-                    for j, operand in enumerate(self.parent_tensors):
-                        if j != i:
-                            dummy_shapes.append(operand.data.shape)
-                    dummy_arrays = [xp.zeros(shape) for shape in dummy_shapes]
-                    # contract_path returns (path, path_info)
-                    path, _ = opt_einsum.contract_path(
-                        grad_equation, *dummy_arrays, optimize='auto'
-                    )
-                    self.backward_paths[grad_equation] = path
-                grad = opt_einsum.contract(
-                    grad_equation, *grad_operands,
-                    backend='cupy' if xp.__name__ == 'cupy' else 'numpy',
-                    optimize=self.backward_paths[grad_equation]
-                )
-            else:
+            try:
                 grad = xp.einsum(grad_equation, *grad_operands, optimize=self.optimize)
-            grad = self._handle_broadcasting(grad, self.operand_shapes[i])
-            grads.append(grad)
+                grad = self._handle_broadcasting(grad, self.operand_shapes[i])
+                grads.append(grad)
+            except Exception as e:
+                grads.append(xp.zeros_like(self.input_operands[i]))
         return tuple(grads)
     def _handle_broadcasting(self, grad, original_shape):
         if grad.shape == original_shape:
@@ -192,7 +158,6 @@ class EinSum(Function, Module):
             if orig_dim == 1 and grad_dim > 1:
                 grad = grad.sum(axis=i, keepdims=True)
         return grad
-    
 
 class Transpose(Function, Module):
     name = "Transpose"
