@@ -130,8 +130,6 @@ class Cast(Function):
         x = self.parent_tensors[0]
         if not x.requires_grad:
             return None
-        if grad_output is None:
-            return None
         if grad_output.dtype == self.original_dtype:
             return grad_output
         return grad_output.astype(self.original_dtype, copy=False)
@@ -173,7 +171,7 @@ class Pad(Function, Module):
         if not hasattr(self, "pad_width"):
             raise RuntimeError("pad_width not set; call forward() first.")
         del parent_tensors[0].data  # free parent tensor data to save memory
-        ng.flush(gc=False)
+        # ng.flush(gc=False)
         slices = tuple(slice(l, None if u == 0 else -u) for (l, u) in self.pad_width)
         parent_tensors[0].data = output_tensor.data[slices]
 
